function A = assemMat_generic(Mesh,EHandle,locglobmap,varargin)
% Generic LehrFEM assembly function for a finite element Galerkin 
% matrix, see also \lectsect{sec:assalg} and, in particular 
% \lectref[Code]{mc:assFE} and \lectref[Code]{mc:assemMat_QFE}. 

% \texttt{Mesh} is supposed to contain a LehrFEM mesh data structure, see 
% \lectsect{sec:meshdata}, maybe of basic type only, \emph{cf.} 
% \lectref[Example]{ex:matlabmeshdata}

% \texttt{EHandle} is a handle to a function performing the computation of 
% the element (stiffness) matrices, called \texttt{getElementMatrix} in 
% \lectref[Code]{mc:assFE}. This function expects a $3\times 2$-matrix 
% argument, whose columns contain the coordinates of the vertices 
% of the triangle whoses element matrix is requested. 

% \texttt{locaglobmap} is a handle to a MATLAB function \texttt{locglobmap} that 
% realizes the local$\to$global index mapping from  \lectref[Eq.]{eq:lgm}.

A = sparse(1, 1);

% number of triangles in the mesh
nElements = size(Mesh.Elements, 1);

% loop over all cells: {\color{red}cell-oriented assembly}
for i = 1:nElements
    % obtain vertex coordinates
    Vertices = Mesh.Coordinates(Mesh.Elements(i,:),:);

    % compute element matrix ($\to$ \lectref[Def.]{def:locstiff}) for current 
    % triangle
    Aloc = EHandle(Vertices, varargin{:});

    % number of local shape functions
    Qk = size(Aloc, 1);

    % obtain a row vector of global indices associated with the 
    % local shape functions, see the discussion of the {\color{red}local$\to$global}
    % {\color{red}index map} in \lectsect{sec:assalg} and \lectref[Eq.]{eq:lgm}
    idx = locglobmap(i, 1:Qk);

    % increase the size of A if we must
    if max(idx) > size(A,1)
        A(max(idx),max(idx)) = 0;
    end

    % Add element matrix to a suitable sub-matrix of the Galerkin 
    % matrix in the spirit of \lectref[Eq.]{ASS}
    A(idx,idx) = A(idx,idx) + Aloc; 
end
